<?php
/**
 * Copyright © Aedes Web Solutions. All rights reserved.
 */
namespace Lhv\HirePurchase\Model\Response;

use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Model\Order;

/**
 * Class Processor
 */
class Processor
{
    /**
     * @var \Magento\Framework\DB\TransactionFactory
     */
    protected $transactionFactory;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterface
     */
    protected $orderInterface;

    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\OrderSender
     */
    protected $orderSender;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var \Magento\Sales\Model\OrderFactory
     */
    protected $orderFactory;

    /**
     * @var QuoteRepository
     */
    protected $quoteRepository;

    /**
     * @param \Magento\Framework\DB\TransactionFactory $transactionFactory
     * @param \Magento\Sales\Api\Data\OrderInterface $orderInterface
     * @param \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     * @param \Magento\Quote\Model\QuoteRepository $quoteRepository
     */
    public function __construct(
        \Magento\Framework\DB\TransactionFactory $transactionFactory,
        \Magento\Sales\Api\Data\OrderInterface $orderInterface,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Quote\Model\QuoteRepository $quoteRepository
    ) {
        $this->transactionFactory = $transactionFactory;
        $this->orderInterface = $orderInterface;
        $this->orderSender = $orderSender;
        $this->checkoutSession = $checkoutSession;
        $this->orderFactory = $orderFactory;
        $this->quoteRepository = $quoteRepository;
    }

    /**
     * Processes the order
     *
     * @param array $data
     * @return \Magento\Sales\Model\Order
     * @throws LocalizedException
     * @throws bool
     */
    public function processOrder($data)
    {
        /** @var $order \Magento\Sales\Model\Order */
        $order = $this->orderInterface->load($data['VK_STAMP']);

        //Ignore duplicate processing, since we are dealing with full captures
        if ($order->hasInvoices()) {
            return $order;
        }

        if (!$order->getEmailSent()) {
            $this->orderSender->send($order);
        }

        $vkData = simplexml_load_string($data['VK_DATA']);
        $contractNumber = $vkData->ContractNumber;
        $order->getPayment()->setTransactionId($contractNumber);

        /** @var $invoice \Magento\Sales\Model\Order\Invoice */
        $invoice = $order->prepareInvoice();

        if (!$invoice) {
            throw new LocalizedException(__('Invoice could not be created.'));
        }

        $invoice->register();
        $invoice->capture();

        $order = $invoice->getOrder();
        $payment = $order->getPayment();
        $transaction = $payment->getCreatedTransaction();

        $this->transactionFactory->create()
            ->addObject($payment)
            ->addObject($transaction)
            ->addObject($invoice)
            ->addObject($order)
            ->save();

        $quote = $this->quoteRepository->get($order->getQuoteId());
        $quote->setReservedOrderId($order->getRealOrderId())->setIsActive(false);
        $this->quoteRepository->save($quote);

        return $order;
    }

    /**
     * Cancels the last order, restores the quote
     *
     * @return void
     */
    public function cancelOrderAndRestoreQuote()
    {
        $orderId = $this->checkoutSession->getLastOrderId();

        /** @var \Magento\Sales\Model\Order $order */
        $order = $orderId ? $this->orderFactory->create()->load($orderId) : false;

        if ($order && $order->getId() && $order->getQuoteId() == $this->checkoutSession->getQuoteId()) {
            $order->cancel()->save();
            $this->restoreQuote($order);
        } else if ($order) {
            $order->cancel()->save();
        }
    }

    /**
     * Restore the quote
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    protected function restoreQuote($order)
    {
        $quote = $this->quoteRepository->get($order->getQuoteId())->setIsActive(true);
        $this->quoteRepository->save($quote);

        $this->checkoutSession
            ->unsLastQuoteId()
            ->unsLastSuccessQuoteId()
            ->unsLastOrderId()
            ->unsLastRealOrderId();
    }
}
