<?php
/**
 * Copyright © Aedes Web Solutions. All rights reserved.
 */
namespace Lhv\HirePurchase\Model\Security;

/**
 * Class MacCalculation
 */
class MacCalculation
{
    /**
     * Algorithm for signature signing
     */
    const ALGORITHM = OPENSSL_ALGO_SHA1;

    /**
     * @var \Lhv\HirePurchase\Gateway\Config\Config
     */
    protected $config;

    /**
     * List of ignored request array entries
     *
     * @var array
     */
    protected $ignoreList = ['VK_MAC', 'VK_ENCODING', 'VK_LANG'];

    /**
     * @param \Lhv\HirePurchase\Gateway\Config\Config $config
     */
    public function __construct(
        \Lhv\HirePurchase\Gateway\Config\Config $config
    ) {
        $this->config = $config;
    }

    /**
     * Calculate the message authentication code from the request fields, base_64 encoded
     *
     * @param array $fields
     * @return string
     */
    public function calculateMac($fields)
    {
        $macFormattedString = $this->getMacFormattedString($fields);
        openssl_sign($macFormattedString, $signature, $this->getPrivateKeyResource(), self::ALGORITHM);
        return base64_encode($signature);
    }

    /**
     * Verify the message authentication code from the response fields
     *
     * @param array $fields
     * @return int
     */
    public function verifyMac($fields)
    {
        $publicKey = openssl_pkey_get_public($this->config->getPublicKey());

        return openssl_verify(
            $this->getMacFormattedString($fields),
            base64_decode($fields['VK_MAC']),
            $publicKey,
            self::ALGORITHM
        );
    }

    /**
     * Get a formatted string for mac calculation
     *
     * Values are prepended with their length and added behind each other.
     *
     * @param array $fields
     * @return string
     */
    protected function getMacFormattedString($fields)
    {
        $fieldsString = '';
        $encoding = $this->getEncoding($fields);

        foreach ($fields as $fieldName => $value) {
            if (!$this->isIgnoredField($fieldName)) {
                $fieldsString .= sprintf("%s%s", $this->getFieldValueLength($value, $encoding), $value);
            }
        }

        return $fieldsString;
    }

    /**
     * Get the private key resource for mac calculation
     *
     * @return bool|resource
     */
    protected function getPrivateKeyResource()
    {
        $password = $this->config->getPrivateKeyPassword();

        if (empty($password)) {
            $privateKeyResource = openssl_pkey_get_private($this->config->getPrivateKey());
        } else {
            $privateKeyResource = openssl_pkey_get_private($this->config->getPrivateKey(), $password);
        }

        return $privateKeyResource;
    }

    /**
     * Get the specified encoding from the request fields, defaults to UTF-8 if field is missing
     *
     * @param array $fields
     * @return string
     */
    protected function getEncoding($fields)
    {
        if (isset($fields['VK_ENCODING'])) {
            $encoding = $fields['VK_ENCODING'];
        } else {
            $encoding = 'UTF-8';
        }

        return $encoding;
    }

    /**
     * Check whether the field is ignored in mac calculation logic
     *
     * @param string $fieldName
     * @return bool
     */
    protected function isIgnoredField($fieldName)
    {
        return in_array($fieldName, $this->ignoreList);
    }

    /**
     * Get the length of the request field's value
     *
     * Returns a string prefixed with zeroes if the length is under 1000 characters (eg 005 or 12345).
     *
     * @param string $value
     * @param string $encoding
     * @return string
     */
    protected function getFieldValueLength($value, $encoding = 'UTF-8')
    {
        return str_pad(mb_strlen($value, $encoding), 3, '0', STR_PAD_LEFT);
    }
}